#!/usr/bin/bash

degree=4

awk -vdegree=$degree '

BEGIN {
    printf ("degree = %s\n\n", degree);

    printf ("define display_e(i) {\n");
    for (i=0; i<degree; ++i)
        printf("  print \"%s\",x%s[i]\n", (i==0) ? "[" : ",", i);
    printf ("  print \"]\\n\"\n");
    printf ("}\n\n");

    printf ("define reduce_e(o) {\n");
    printf("  t = denominator[o]\n");
    for (i=0; i<degree; ++i)
        printf("  t = gcd(x%s[o],t)\n", i);
    for (i=0; i<degree; ++i)
        printf("  x%s[o] /= t\n", i);
    printf("  denominator[o] /= t\n");
    printf ("}\n\n");

    printf ("define extract_beb(o,i1,i2) {\n");
    printf("  j = numerator[i2]\n");
    printf("  if (j < 1 || degree < j || denominator[i2] != 1) {\n");
    printf("    ignore = error_message(7)\n");
    printf("    return\n  }\n");
    for (i=0; i<degree; ++i)
        printf("  if (j == %s) numerator[o] = x%s[i1]\n", i+1, i);
    printf("  denominator[o] = denominator[i1]\n");
    printf("  ignore = reduce_b(o)\n");
    printf ("}\n\n");

    printf ("define convert_eb(o,i) {\n");
    for (i=0; i<degree; ++i)
        printf("  x%s[o] = %s\n", i, (i==0) ? "numerator[i]" : "0");
    printf("  denominator[o] = denominator[i]\n");
    printf ("}\n\n");

    printf ("define reset_e(o) {\n");
    for (i=0; i<degree; ++i)
        printf("  x%s[o] = 0\n", i);
    printf("  denominator[o] = 1\n");
    printf ("}\n\n");

    printf ("define initialize_ebl(o,i1,i2) {\n");
    for (i=0; i<degree; ++i)
        printf("  x%s[o] *= denominator[i1]\n", i);
    for (i=0; i<degree; ++i)
        printf("  if (i2 == %s) x%s[o] = numerator[i1]*denominator[o]\n", i+1, i);
    printf("  denominator[o] *= denominator[i1]\n");
    printf("  ignore = reduce_e(o)\n");
    printf ("}\n\n");

    printf ("define copy_ee(o,i) {\n");
    for (i=0; i<degree; ++i)
        printf("  x%s[o] = x%s[i]\n", i, i);
    printf("  denominator[o] = denominator[i]\n");
    printf ("}\n\n");

    printf ("define negate_ee(o,i) {\n");
    for (i=0; i<degree; ++i)
        printf("  x%s[o] = - x%s[i]\n", i, i);
    printf("  denominator[o] = denominator[i]\n");
    printf ("}\n\n");

    printf ("define add_eee(o,i1,i2) {\n");
    for (i=0; i<degree; ++i)
        printf("  x%s[o] = x%s[i1]*denominator[i2] + x%s[i2]*denominator[i1]\n", i, i, i);
    printf("  denominator[o] = denominator[i1] * denominator[i2]\n");
    printf("  ignore = reduce_e(o)\n");
    printf ("}\n\n");

    printf ("define subtract_eee(o,i1,i2) {\n");
    for (i=0; i<degree; ++i)
        printf("  x%s[o] = x%s[i1]*denominator[i2] - x%s[i2]*denominator[i1]\n", i, i, i);
    printf("  denominator[o] = denominator[i1] * denominator[i2]\n");
    printf("  ignore = reduce_e(o)\n");
    printf ("}\n\n");

    printf ("define multiply_eeb(o,i1,i2) {\n");
    for (i=0; i<degree; ++i)
        printf("  x%s[o] = x%s[i1] * numerator[i2]\n", i, i);
    printf("  denominator[o] = denominator[i1] * denominator[i2]\n");
    printf("  ignore = reduce_e(o)\n");
    printf ("}\n\n");

    printf ("define multiply_eee(o,i1,i2) {\n");
    n = 0
    for (i=0; i<degree; ++i)
        for (j=0; j<degree; ++j)
            for (k=0; k<degree; ++k)
	        printf("  x%s[o] %s= mult[%s]*x%s[i1]*x%s[i2]\n", i, (j || k) ? "+" : " ",n++, j, k);
    printf("  denominator[o] = denominator[i1] * denominator[i2]\n");
    printf("  ignore = reduce_e(o)\n");
    printf ("}\n\n");

    printf ("define conjugate_eeb(o,i1,i2) {\n");
    printf ("  if (numerator[i2] < 0 || degree <= numerator[i2] || denominator[i2] != 1) {\n");
    printf ("    ignore = error_message(8)\n");
    printf ("    return\n");
    printf ("  }\n");
    printf("  offset = numerator[i2] * degree^2\n");
    n = 0
    for (i=0; i<degree; ++i)
        for (j=0; j<degree; ++j)
	    printf("  x%s[o] %s= conj[offset+%s]*x%s[i1]\n", i, j ? "+" : " ",n++, j);
    printf("  denominator[o] = denominator[i1]\n");
    printf("  ignore = reduce_e(o)\n");
    printf ("}\n\n");

    printf ("define locate(i) {\n");
    printf ("  if (denominator[i] != 1) {\n");
    printf ("    ignore = error_message(13)\n");
    printf ("    return -1\n");
    printf ("  }\n");
    printf ("  error = 0\n");
    for (i=0; i<degree; ++i)
        printf("  if (x%s[i]!=0 && x%s[i]!=1) error=1\n", i, i);
    printf ("  if (error == 1) {\n");
    printf ("    ignore = error_message(14)\n");
    printf ("    return -1\n");
    printf ("  }\n");
    printf ("  count = 0\n");
    for (i=0; i<degree; ++i)
        printf("  if (x%s[i]==1) ++count\n", i);
    printf ("  if (count != 1) {\n");
    printf ("    ignore = error_message(15)\n");
    printf ("    return -1\n");
    printf ("  }\n");
    printf ("  loc = 0\n");
    for (i=0; i<degree; ++i)
        printf("  if (x%s[i]==1) loc=%s\n", i, i);
    printf ("  return (loc)\n");
    printf ("}\n\n");

    printf ("define initialize_mult(l_arg,r_arg,res) {\n");
    printf ("  n = locate(l_arg)\n");
    printf ("  if (n == -1) return\n");
    printf ("  m = locate(r_arg)\n");
    printf ("  if (m == -1) return\n");
    printf ("  offset = degree*(n*degree + m);\n");
    for (i=0; i<degree; ++i)
        printf("  mult[offset+%s] = x%s[res];\n", i, i);
    printf ("}\n\n");

    printf ("define initialize_conj(l_arg,r_arg,res) {\n");
    printf ("  n = locate(l_arg)\n");
    printf ("  if (n == -1) return\n");
    printf ("  m = numerator(r_arg)\n");
    printf ("  if (denominator[r_arg] != 1) return\n");
    printf ("  if (m < 0 || degree <= m) return\n");
    printf ("  offset = degree*(n*degree + m);\n");
    for (i=0; i<degree; ++i)
        printf("  conj[offset+%s] = x%s[res];\n", i, i);
    printf ("}\n\n");

    printf("for (i=0; i<degree^3; ++i) {\n");
    printf("  mult[i] = 0\n  conj[i] = 0\n}\n");
}

' < /dev/null
